/* Encode all video files in a directory to xvid (*.avi) using a 2 pass method and mencoder 

   (c) Andreas Ludwig 2008 - this is freeware, provided AS IS. 
   Mencoder might overwrite files on your harddrive, so make sure you test
   and/or know what you are doing ;o)
   Contact: Andreas-Ludwig.info

*/

/* Command line arguments:
	1: Directory to read original file from (wrap directories with spaces in quotation marks)
	2: Directory to write converted files into (wrap directories with spaces in quotation marks)
	3: Bitrate (video)
	4: Optional: Cropping range (w:h:x:y) 
		You can use mplayer -rectangle=w:h:x:y to test
		(see mencoder manpage for detailed description)
	5: Optional: Size for final scaling (w:h:c:p)
		Make sure H and V are multiples of 16 and
		X and Y are at least even (or multiples of 4
		if you have interleaved original material
		ATTENTION: Option 5 ONLY with option 4
	6: Optional: Any further text will just be passed on as additional option
*/

/* Adaptations of the script:
	1. Here you need to set the correct path to mencoder */
MEncPath = 'F:\Anwendungen\mplayer2\mencoder.exe'
/*	Use MEncPath = 'mencoder.exe' if mencoder is in a directory 
	on your PATH in config.sys */

/*	2. Here you can set the matching expression for files to be transformed */
FileExp = '*.mpg'
/*	No sanity check is done on this pattern, so make sure you know
	what you are doing or else MEncoder will fail on you 
	Use FileExp = '*' to read in any file in SourceDir */	 	

/*	3. Bitrate for lamemp3 audio encoding (using vbr=abr) */
ABitRate = 160

/*	ATTENTION: 	Make sure you have a couple of megabites free in the 
			directory you store the script in so it can save the 
			log of the analysis in the first pass 
			(no actual output will be created though)

	Good tutorials on using mencoder for various things:
	- http://gentoo-wiki.com/HOWTO_Mencoder_Introduction_Guide
	- http://gentoo-wiki.com/HOWTO_Rip_DVD_mencoder

	Some more info on cutting and scaling
	- http://forum.doom9.org/archive/index.php/t-71877.html
*/

call RxFuncAdd 'SysLoadFuncs', 'REXXUTIL', 'SysLoadFuncs'
call SysLoadFuncs

/* Get the script name and path */
parse source OpSys CallType ScriptPathAndName

/* Get the arguments to this scipt - using arg() to handle paths with blanks correctly */
/* SourceDir */
if left(arg(1),1)='"' then /* search for next quotation mark and regard what's in between as path */
   do
      SourceDir = substr(arg(1),2,pos('"',arg(1),2) - 2)
      NextPos = length(SourceDir) + 4
   end
else /* Use argument string up until the next blank */
   do
      SourceDir = left(arg(1),pos(' ',arg(1)) - 1)
      NextPos = length(SourceDir) + 2
   end
say 'SourceDir: 'SourceDir
/* say NextPos */
/* OutputDir */
if substr(arg(1),NextPos,1)='"' then /* search for next quotation mark and regard what's in between as path */
   do
      OutputDir = substr(arg(1),NextPos+1,pos('"',arg(1),NextPos+1) - NextPos - 1)
      NextPos = NextPos + length(OutputDir) + 3
   end
else /* Use argument string up until the next blank */
   do
      OutputDir = substr(arg(1),NextPos,pos(' ',arg(1),NextPos) - NextPos)
      NextPos = NextPos + length(OutputDir) + 1
   end
say 'OutputDir: 'OutputDir
/* say NextPos */
/* VBitRate */
/* Take the string to the next blank as bitrate (or the rest of the argument if there is none ;o) */
if pos(' ',arg(1),NextPos) = 0 then
   VBitRate = substr(arg(1),NextPos)
else
   VBitRate = substr(arg(1),NextPos,pos(' ',arg(1),NextPos) - NextPos)
NextPos = NextPos + length(VBitRate) + 1
say 'VBitRate: 'VBitRate
/* Initialize optional arguments to empty strings */
VFOpt = ''
AuxArg = ''
/* CropRange */
/* In case we did not reach the end of the string yet - use next/remaining part as crop range */
if NextPos < length(arg(1)) then 
   do
      if pos(' ',arg(1),NextPos) = 0 then
         CropRange = substr(arg(1),NextPos)
      else
         CropRange = substr(arg(1),NextPos,pos(' ',arg(1),NextPos) - NextPos)
      NextPos = NextPos + length(CropRange) + 1
      say 'CropRange: 'CropRange
      VFOpt = insert('-vf crop=',CropRange)
   end
else
   say 'No cropping.'
/* Scaling */
/* In case we did not reach the end of the string yet - use next/remaining part as scaling size */
if NextPos < length(arg(1)) then 
   do
      if pos(' ',arg(1),NextPos) = 0 then
         Scaling = substr(arg(1),NextPos)
      else
         Scaling = substr(arg(1),NextPos,pos(' ',arg(1),NextPos) - NextPos)
      NextPos = NextPos + length(Scaling) + 1
      say 'Scaling option: 'Scaling
      VFOpt = insert(VFOpt,insert(',scale=',Scaling))
   end
else
   say 'No scaling.'
/* Additional options */
/* In case we still did not reach the end of the string - just pass the rest on as auxilliary option to mencode */
if NextPos < length(arg(1)) then AuxArg = substr(arg(1),NextPos)

if (arg() = 0) | (arg(1) = 'help') | (arg(1) = '-h') | (arg(1) = '/h') then do
   say 'Usage: ' ScriptPathAndName 'SourceDir OutputDir VBitRate (CropRange as w:h:x:y (Scaling as w:h:c:p))'
   exit 1
end

/* Read in complete file information  for source dir */
call SysFileTree insert(SourceDir,insert('\',FileExp)), 'Originals', F

/* Display files with complete path and file info to get user confirmation */
say 'Files to be converted:'
say ''
do i=1 to Originals.0
   say Originals.i
end i
say ''

say 'Continue (Y/n)?'
pull ContinueHere
say ''

if ContinueHere='N' then exit 1

/* Read in the filenames only now */
call SysFileTree insert(SourceDir,insert('\',FileExp)), 'Originals', FO

/* Now do the actual transformation into 2-pass xvid avis */
do i=1 to Originals.0
   /* Generate new file name and path out of original filename */
   /* Check if file has extention (*.mpg, *.mpeg, etc.) */
   LastDotPos = pos('.',Originals.i,lastpos('\',Originals.i)+1)
   /* Extract filename without extension */
   if LastDotPos = 0 then 
      NewFile = substr(Originals.i,lastpos('\',Originals.i)+1)
   else 
      NewFile = substr(Originals.i,lastpos('\',Originals.i)+1, LastDotPos - lastpos('\',Originals.i) - 1)
   /* Append .avi extension */
   NewFile = insert(NewFile,'.avi')
   NewFile = insert(OutputDir,insert('\',NewFile))
   say 'File 'i' of 'Originals.0':'
   say Originals.i' -> 'NewFile
   /* Uncomment the next line to pause before every file */
   /* pull dummy */
   MEncPath' "'Originals.i'" -ovc xvid -xvidencopts pass=1 -oac copy 'VFOpt' -o /dev/null 'AuxArg
   MEncPath' "'Originals.i'" -ovc xvid -xvidencopts pass=2:bitrate='VBitRate' -oac mp3lame -lameopts vbr=3:br='ABitRate' 'VFOpt' -o "'NewFile'" 'AuxArg
   say''
end i

say 'Finished.'
pull dummy
